@@ -1,11 +1,13 @@
 import os 
 import json
 import datetime
+import requests
 from rocrateValidator.utils import Result as Result
 import zipfile
 import rocrate.utils as utils
 import rocrate.rocrate as rocrate
 from datetime import datetime
+import rocrateValidator.workflow_extension as we
 import pytest
 
 correct_value = {
@@ -51,7 +53,7 @@ def file_descriptor_check(tar_file, extension):
     """
     NAME = "File descriptor check"
     error_message = {
-        "DescriptorError": "entity property of self descriptor is incorrect"
+        "DescriptorError": "Semantic Error: Invalid file descritor."
     }
     
     with open (os.path.join(tar_file, "ro-crate-metadata.json"), 'r') as file:
@@ -73,14 +75,26 @@ def datetime_valid(dt_str):
         return False
     return True
 
-def dataset_property(entity, type):
+def dateTime_property(entity):
     try:
-        id = utils.get_norm_value(entity, '@id')[0]
-    except IndexError:
+        date = utils.get_norm_value(entity, "datePublished")[0]
+        if datetime_valid(date):
+            return True
+        else:
+            print("asdfd")
+            return False
+    except:
+        return "ssdfa"
+
+def type_property(entity, type):
+    try: 
+        id = utils.get_norm_value(entity, "@id")[0]
+    except:
         return False
-    if type[0] == ['Dataset'] and id.endswith('/'):
+    if type[0] == "Dataset" and id.endswith("/"):
         return True
-    return False
+    else:
+        return False
             
 def direct_property_check(tar_file, extension):
     """\
@@ -90,25 +104,25 @@ def direct_property_check(tar_file, extension):
     """
     NAME = "Direct property check"
     error_message = {
-        "DateError": "datePublished at {} is not in ISO 8601 date format",
-        "DirectPropError": "Directory property of RO-Crate is wrong"
+        "IDError":"Semantic Error: Invalid Direct Property ID. It MUST end with / and SHOULD be ./",
+        "TypeError": "Semantic Error: Invalid Type Value at Direct Property ./",
+        "DateError": "Semantic Error: datePublished of Direct Property ./ is not in ISO 8601 date format or not Provided"
     }
-    
-    with open (os.path.join(tar_file, "ro-crate-metadata.json"), 'r') as file:
-        metadata = json.load(file)
-        graph = metadata['@graph']
-    
     ### check each entity in @graph of metadata, each type must be Dataset and datePublished has to be in ISO format
-    for entity in graph: 
-        type = utils.get_norm_value(entity, '@type')
-        if dataset_property(entity, type):
-            for _ in utils.get_norm_value(entity, 'datePublished'):
-                if datetime_valid(_) == True:
-                    return Result(NAME)
-                else:
-                    return Result(NAME, code = -1, message = error_message["DateError"].format(_) )
-                
-    return Result(NAME, code = -1, message = error_message["DirectPropError"])
+    context, metadata = rocrate.read_metadata(os.path.join(tar_file, "ro-crate-metadata.json"))
+    
+    try:
+        direct_property_entity = metadata["./"]
+        type = utils.get_norm_value(direct_property_entity, "@type")
+        if type_property(direct_property_entity, type):
+            if dateTime_property(direct_property_entity):
+                return Result(NAME)
+            else:
+                return Result(NAME, code = -1, message = error_message["DateError"])
+        else:
+            return Result(NAME, code = -1, message = error_message["TypeError"])
+    except:
+        return Result(NAME, code = -1, message = error_message["IDError"])
 
 
 ### the value of type must either be string of expected type or a list with expected type in elements. 
@@ -134,7 +148,7 @@ def referencing_check(tar_file, extension):
     
     NAME = "Referencing check"
     error_message = {
-        "ReferError": "The referencing {} is wrong."
+        "ReferError": "Semantic Error: The referencing {} is wrong."
     }
     
     ### Create a dictionary to store the referencing check result
@@ -191,8 +205,8 @@ def encoding_check(tar_file, extension):
     
     NAME = "Encoding check"
     error_message = {
-        "EncodingError" : "Encoding in {} is wrong",
-        "TypeError": "The value of @type of {} entity is incorrect."
+        "EncodingError" : "Semantic Error: Encoding in {} is wrong",
+        "TypeError": "Semantic Error: The value of @type of {} entity is incorrect."
     }
     
     ### Create a dictionary to store the encoding check result
@@ -213,7 +227,8 @@ def encoding_check(tar_file, extension):
     ### If any of the value in the dictionary are false which should be a list, then return false
     for values in encoding_result.values():
         if isinstance(values, list):
-            return Result(NAME, code = -1, message = error_message["EncodingError"].format(list(encoding_result.keys())[list(encoding_result.values()).index([values[0], values[1]])]))
+            return Result(NAME, code = -1, message = values[1])
+            # return Result(NAME, code = -1, message = error_message["EncodingError"].format(list(encoding_result.keys())[list(encoding_result.values()).index([values[0], values[1]])]))
     
     return Result(NAME)
 
@@ -260,9 +275,9 @@ def webbased_entity_check(tar_file, extension):
     
     NAME = "Web-based data entity check"
     error_message = {
-        "UrlError": "Web-based data entity {} should be a downloadable url",
-        "DateError": "sdDatePublished of web based data entity {} is not provided or incorrect",
-        "TypeError": "the value of @type of {} has to be DataDownload"
+        "UrlError": "Semantic Error: Invalid ID at {}. It `should be a downloadable url",
+        "DateError": "Semantic Error: Invalid sdDatePublished {} or Not Provided",
+        "TypeError": "Semantic Error: Invalid @type value of {}."
     }
     webbased_result = {}
     
@@ -308,8 +323,8 @@ def person_entity_check(tar_file, extension):
     
     NAME = "Person entity check"
     error_message = {
-        "PersonError": "Person entity {} is incorrect",
-        "TypeError": "The value of @type of {} is incorrect"
+        "PersonError": "Semantic Error: Invalid Person entity {}",
+        "TypeError": "Semantic Error: Invalid @type value of {}"
     }
     warning_message = {
         "OrganizationAuthor" : "WARNING: The Author {} is an Organization"
@@ -397,7 +412,7 @@ def organization_check(tar_file, extension):
     
     NAME = "Organization entity check"
     error_message = {
-        "OrganizationError": "Organization contextual entity {} is incorrect"
+        "OrganizationError": "Semantic Error: Invalid Organization contextual entity {}"
     }
     
     ### dictionary to store the checking result
@@ -434,7 +449,7 @@ def contact_info_check(tar_file, extension):
     
     NAME = "Contact information check"
     error_message = {
-        "TypeError": "Invlaid Type Value at {}"
+        "TypeError": "Semantic Error: Invlaid Type Value at {}"
     }
     
     ###dictionary to store the checking result
@@ -463,8 +478,8 @@ def citation_check(tar_file, extension):
     
     NAME = "Citation property check"
     error_message = {
-        "TypeError": "Invalid Type Value at {}",
-        "IDError": "Invalid ID Value at {}"
+        "TypeError": "Semantic Error: Invalid Type Value at {}",
+        "IDError": "Semantic Error: Invalid ID Value at {}"
     }
     
     ### Dict to store the checking result
@@ -494,8 +509,8 @@ def publisher_check(tar_file, extension):
     
     NAME = "Publisher property check"
     error_message = {
-        "TypeError": "Invalid Type Value at {}",
-        "ReferencingError": "Invalid Referencing {} or Not Provided"
+        "TypeError": "Semantic Error: Invalid Type Value at {}",
+        "ReferencingError": "Semantic Error: Invalid Referencing {} or Not Provided"
     }
     
     publisher_result = {}
@@ -523,8 +538,8 @@ def funder_check(tar_file, extension):
     
     NAME = "Funder property check"
     error_message = {
-        "TypeError": "Invalid Type Vlaue at {}",
-        "ReferencingError": "Reference Entity {} Missing"
+        "TypeError": "Semantic Error: Invalid Type Vlaue at {}",
+        "ReferencingError": "Semantic Error: Reference Entity {} Missing"
     }
     
     funder_result = {}
@@ -547,16 +562,17 @@ def funder_check(tar_file, extension):
 
 
 def upd_licenseRlt(entity, license, metadata, licensing_result, error_message):
-    if utils.is_url(license[0]) and utils.get_norm_value(entity, "@id")[0] != "./" and utils.get_norm_value(entity, "@id"[0] != "ro-crate-metadata.json"):
+    if utils.is_url(license[0]) and utils.get_norm_value(entity, "@id")[0] != "./" and utils.get_norm_value(entity, "@id")[0] != "ro-crate-metadata.json":
         try:
             license_entity = metadata[license[0]]
+            type = utils.get_norm_value(license_entity, "@type")
+            if type[0] == "CreativeWork":
+                licensing_result[license[0]] = True
+            else:
+                licensing_result[license[0]] = [False, error_message["TypeError"].format(license[0])]
         except KeyError:
             licensing_result[license[0]] = [False, error_message["ReferencingError"]]
-        type = utils.get_norm_value(license_entity, "@type")
-        if type[0] == "CreativeWork":
-            licensing_result[license[0]] = True
-        else:
-            licensing_result[license[0]] = [False, error_message["TypeError"].format(license[0])]
+        
     else:
         licensing_result[license[0]] = [False, error_message["IDError"].format(license[0])]
 
@@ -571,9 +587,9 @@ def licensing_check(tar_file, extension):
     
     NAME = "Licensing property check"
     error_message = {
-        "TypeError":"Invalid Type Value at {}", 
-        "IDError" : "Invalid ID Value at {}. It must be an URL.", 
-        "ReferencingError": "Invalid Referencing or NOT Provided"
+        "TypeError":"Semantic Error: Invalid Type Value at {} or not provided.", 
+        "IDError" : "Semantic Error: Invalid ID Value at {}. It must be an URL.", 
+        "ReferencingError": "Semantic Error: Invalid Referencing or NOT Provided"
     }
     
     licensing_result = {}
@@ -616,8 +632,8 @@ def places_check(tar_file, extension):
     
     NAME = "Places property check"
     error_message = {
-        "TypeError":"Invalid Type Value at {}",
-        "ReferencingError": "Invalid Refernencing or Not Provided"
+        "TypeError":"Semantic Error: Invalid Type Value at {}",
+        "ReferencingError": "Semantic Error: Invalid Refernencing or Not Provided"
     }
     
     geo_result = {}
@@ -659,7 +675,7 @@ def time_check(tar_file, extension):
     
     NAME = "Time property check"
     error_message = {
-        "TypeError": "Invalid Type Value at {}"
+        "TypeError": "Semantic Error: Invalid Type Value at {}"
     }
     
     time_result = {}
@@ -732,11 +748,11 @@ def scripts_and_workflow_check(tar_file, extension):
     """
 
     NAME = "Scripts and workflow check"
-    wkfext_path = '/Users/xuanqili/Desktop/ro-crate-validator-py/src/workflow_extension.txt'
+    # wkfext_path = '/Users/xuanqili/Desktop/ro-crate-validator-py/src/workflow_extension.txt'
 
     error_message = {
-        "WorkflowError":"Scripts and Workflow is Wrong",
-        "TypeError": " @type for workflow file {} must have File, SoftwareSourceCode and ComputationalWorkflow as value."
+        "WorkflowError":"Semantic Error: Scripts and Workflow is Wrong",
+        "TypeError": "Semantic Error: Invalid @type value for workflow file {}. It must have File, SoftwareSourceCode and ComputationalWorkflow as value."
     }
     warning_message = {
         "UnrecognizedWkf" : "WARNING: {} is not a recognised workflow extension. Please raise an issue at GitHub: <https://github.com/ResearchObject/ro-crate-validator-py/issues>."
@@ -750,8 +766,9 @@ def scripts_and_workflow_check(tar_file, extension):
     ### check if recognised workflow file meets the requirments
     for entity in metadata.values():
         id_ = utils.get_norm_value(entity, "@id")[0]
-        with open (wkfext_path, "r") as file:
-            extension_set = file.read().splitlines()
+        extension_set = list(we.get_workflow_extension())
+        # with open (wkfext_path, "r") as file:
+        #     extension_set = file.read().splitlines()
         recognisedWkf_upd(extension_set, entity, workflow_result, id_, error_message)
     
     ### check unrecognised workflow file with ComputaionalWorkflow in its @type
